@@ -3,6 +3,8 @@
  */
 package com.typesafe.config.impl
 
+import java.math.BigInteger
+
 import org.junit.Assert._
 import org.junit._
 import com.typesafe.config.ConfigMemorySize
@@ -22,4 +24,10 @@ class ConfigMemorySizeTest extends TestUtils {
         val kilobyte = ConfigMemorySize.ofBytes(1024)
         assertEquals(1024, kilobyte.toBytes)
     }
+
+    @Test
+    def testGetBytes() {
+        val yottabyte = ConfigMemorySize.ofBytes(new BigInteger("1000000000000000000000000"))
+        assertEquals(new BigInteger("1000000000000000000000000"), yottabyte.toBytesBigInteger)
+    }
 }
